import { useMarkStore } from "@/store";
import { httpApp } from "@/plugins/axiosApp";
import {
  Setting,
  UISetting,
  HistoryQueryParams,
  MarkStore,
  Group,
  Task,
  CommonResponse,
  MarkResult,
} from "@/types";
import vls from "@/utils/storage";

const markStore = useMarkStore();

const getMarkInfo = () => {
  return vls.get("mark", {});
};

/** 清除评卷任务（之前锁住的任务之类的） */
export async function clearMarkTask() {
  return httpApp.post<void>("/api/mark/clear", {}, { params: getMarkInfo() });
}

/** 获取评卷设置 */
export async function getSetting() {
  return httpApp.post<Setting>(
    "/api/mark/getSetting",
    {},
    { params: getMarkInfo() }
  );
}

/** 获取评卷状态 */
export async function getStatus(
  questionModel: Setting["questionModel"],
  questionId?: string
) {
  return httpApp.post<MarkStore["status"]>(
    "/api/mark/getStatus",
    {},
    {
      params: {
        ...getMarkInfo(),
        questionModel,
        questionId: questionId || undefined,
      },
    }
  );
}

/** 获取评卷分组 */
export async function getGroup() {
  return httpApp.post<Group[]>(
    "/api/mark/getGroup",
    {},
    { params: getMarkInfo() }
  );
}

/** 获取评卷任务 */
export async function getTask(
  questionModel: Setting["questionModel"],
  questionId?: string
) {
  return httpApp.post<Task>(
    "/api/mark/getTask",
    {},
    {
      params: {
        ...getMarkInfo(),
        questionModel,
        questionId: questionId || undefined,
      },
    }
  );
}

/** 更新评卷UI */
export async function updateUISetting(
  mode?: Setting["mode"],
  uiSetting?: UISetting,
  questionModel?: Setting["questionModel"],
  markedQuestionId?: string
) {
  return httpApp.post<void>(
    "/api/mark/updateSetting",
    {},
    {
      params: {
        mode: mode || undefined,
        uiSetting: uiSetting ? JSON.stringify(uiSetting) : undefined,
        questionModel: questionModel || undefined,
        markedQuestionId: markedQuestionId || undefined,
        ...getMarkInfo(),
      },
    }
  );
}

/** 获取评卷历史任务 */
export async function getHistoryTask({
  pageNumber = 1,
  pageSize = 20,
  order = "markerTime",
  sort = "DESC",
  secretNumber = null,
  markerScore = null,
}: HistoryQueryParams) {
  return httpApp.post<Task[]>(
    "/api/mark/getHistory",
    {},
    {
      params: {
        pageNumber,
        pageSize,
        order,
        sort,
        secretNumber,
        markerScore,
        ...getMarkInfo(),
      },
    }
  );
}

/** 保存评卷任务(正常保存) */
export async function saveTask(datas: MarkResult) {
  return httpApp.post<CommonResponse>("/api/mark/saveTask", datas, {
    setGlobalMask: true,
    params: getMarkInfo(),
  });
}

/** 获取用户信息 */
export async function changeUserInfo(name: string, password?: string) {
  return httpApp.post<void>(
    "/api/mark/changeName",
    {},
    {
      params: {
        name,
        password: password || undefined,
      },
    }
  );
}

/** 评卷用户退出 */
export function doLogout() {
  // window.history.go(-1);
  window.close();
}

/** 评卷用户选择分组 */
// ps:已经没用了 3.3.0
export async function doSwitchGroup(markerId: number) {
  return httpApp.post<CommonResponse>(
    "/api/mark/subjectSelect",
    {},
    {
      params: { markerId, ...getMarkInfo() },
    }
  );
}

// TODO: 需要优化: 问题卷按试题设置
/** 评卷用户选择试卷的问题类型 */
export async function doProblemType({
  problemType,
  problemRemark,
}: {
  problemType: string;
  problemRemark?: string;
}) {
  if (!markStore.currentTask?.markResult) return;

  const datas = { ...markStore.currentTask?.markResult };
  datas.problem = true;
  datas.unselective = false;
  datas.problemType = problemType;
  datas.problemRemark = problemRemark || undefined;
  datas.markerScore = null;
  datas.scoreList = [];
  datas.markerTagList = [];
  datas.markerTrackList = [];
  datas.spent = Date.now() - markStore.currentTask.__markStartTime;

  return httpApp.post<CommonResponse>("/api/mark/saveTask", datas, {
    params: getMarkInfo(),
  });
}

/** 评卷用户选择试卷的为未选做 */
export async function doUnselectiveType() {
  if (!markStore.currentTask?.markResult) return;

  const datas = { ...markStore.currentTask?.markResult };
  datas.markerScore = -1;
  datas.scoreList = [];
  datas.markerTagList = [];
  datas.markerTrackList = [];
  datas.spent = Date.now() - markStore.currentTask.__markStartTime;

  return httpApp.post<CommonResponse>("/api/mark/saveTask", datas, {
    params: getMarkInfo(),
  });
}

/** 获取评卷任务 */
export async function getMarkTask(id: string) {
  return httpApp.post<boolean>(
    "/api/admin/mark/task/get_mark_track",
    {},
    { params: { id } }
  );
}
/** 打回评卷任务 */
export async function doRejectTask(params: {
  id: string;
  rejectReason: string;
}) {
  return httpApp.post<boolean>("/api/admin/mark/task/reject", {}, { params });
}
