import { ref, watch } from "vue";
import { useMarkStore } from "@/store";
import { loadImage } from "@/utils/utils";
import EventBus from "@/plugins/eventBus";
import useTrackColor from "./useTrackColor";

// 计算裁切图和裁切图上的分数轨迹和特殊标记轨迹
export default function useSliceTrack() {
  const markStore = useMarkStore();
  const { addTrackColorAttr, addHeaderTrackColorAttr, addTagColorAttr } =
    useTrackColor();

  const maxSliceWidth = ref(0); // 最大的裁切块宽度，图片容器以此为准
  const theFinalHeight = ref(0); // 最终宽度，用来定位轨迹在第几张图片，不包括image-seperator高度

  async function processImage() {
    if (!markStore.currentTask) return;
    const images = [];
    const urls = markStore.currentTask.sheetUrls || [];
    for (const url of urls) {
      const image = await loadImage(url);
      images.push(image);
    }

    maxSliceWidth.value = Math.max(...images.map((i) => i.naturalWidth));
    theFinalHeight.value = Math.max(...images.map((i) => i.naturalHeight));
    markStore.sliceImagesWithTrackList.splice(0);

    const trackLists = (markStore.currentTask.questionList || [])
      .map((q) => {
        return q.headerTrackList?.length
          ? addHeaderTrackColorAttr(q.headerTrackList)
          : addTrackColorAttr(q.markerTrackList || []);
      })
      .flat();
    const tagLists = (markStore.currentTask.questionList || [])
      .map((q) => {
        return q.headerTagList?.length
          ? addTagColorAttr(q.headerTagList)
          : addTagColorAttr(q.markerTagList || []);
      })
      .flat();

    let accumTopHeight = 0;
    let accumBottomHeight = 0;
    for (const url of urls) {
      const indexInSliceUrls = urls.indexOf(url) + 1;
      const image = images[indexInSliceUrls - 1];
      accumBottomHeight += image.naturalHeight;

      const thisImageTrackList = trackLists.filter(
        (t) => t.offsetIndex === indexInSliceUrls
      );
      const thisImageTagList = tagLists.filter(
        (t) => t.offsetIndex === indexInSliceUrls
      );

      markStore.sliceImagesWithTrackList.push({
        url,
        indexInSliceUrls,
        markerTrackList: thisImageTrackList,
        tagList: thisImageTagList,
        originalImageWidth: image.naturalWidth,
        originalImageHeight: image.naturalHeight,
        width: (image.naturalWidth / maxSliceWidth.value) * 100 + "%",
        dx: 0,
        dy: 0,
        accumTopHeight,
      });
      accumTopHeight = accumBottomHeight;
    }
  }

  // should not render twice at the same time
  const renderPaperAndMark = async () => {
    if (!markStore.currentTask) return;
    if (!markStore.isScanImage) return;
    if (markStore.renderLock) {
      console.log("上个任务还未渲染完毕，稍等一秒再尝试渲染");
      await new Promise((res) => setTimeout(res, 1000));
      await renderPaperAndMark();
      return;
    }

    markStore.renderLock = true;
    try {
      markStore.globalMask = true;
      await processImage();
    } catch (error) {
      markStore.sliceImagesWithTrackList.splice(0);
      console.trace("render error ", error);
      // 图片加载出错，自动加载下一个任务
      EventBus.emit("body-render-error");
    } finally {
      markStore.renderLock = false;
      markStore.globalMask = false;
    }
  };

  function addRenderWatch() {
    // 监听 currentTask 变化，重新渲染
    // 在阻止渲染的情况下，watchEffect收集不到 store.currentTask 的依赖，会导致本组件不再更新
    watch(
      () => markStore.currentTask,
      () => {
        setTimeout(renderPaperAndMark, 50);
      }
    );
  }

  return { maxSliceWidth, theFinalHeight, addRenderWatch };
}
