import { useTimers } from "@/setups/useTimers";
import { useMarkStore } from "@/store";
import { onMounted, onUnmounted, watch, computed } from "vue";

export default function useBodyScroll({
  shortCut = false,
  autoScroll = false,
  containerSelector = ".mark-body-container",
}: {
  shortCut?: boolean;
  autoScroll?: boolean;
  container?: string;
} = {}) {
  const { addTimeout } = useTimers();
  const markStore = useMarkStore();

  function getContainer(): HTMLDivElement | null {
    return document.querySelector<HTMLDivElement>(containerSelector);
  }

  // 缩略图定位
  watch(
    () => [markStore.minimapScrollToX, markStore.minimapScrollToY],
    () => {
      const container = getContainer();
      addTimeout(() => {
        if (
          container &&
          typeof markStore.minimapScrollToX === "number" &&
          typeof markStore.minimapScrollToY === "number"
        ) {
          const { scrollWidth, scrollHeight } = container;
          container.scrollTo({
            top: scrollHeight * markStore.minimapScrollToY,
            left: scrollWidth * markStore.minimapScrollToX,
            behavior: "smooth",
          });
        }
      }, 10);
    }
  );

  // 快捷键定位
  if (shortCut) {
    const scrollContainerByKey = (e: KeyboardEvent) => {
      const container = getContainer();
      if (!container) return;

      if (e.key === "w") {
        container.scrollBy({ top: -100, behavior: "smooth" });
      } else if (e.key === "s") {
        container.scrollBy({ top: 100, behavior: "smooth" });
      } else if (e.key === "a") {
        container.scrollBy({ left: -100, behavior: "smooth" });
      } else if (e.key === "d") {
        container.scrollBy({ left: 100, behavior: "smooth" });
      }
    };

    onMounted(() => {
      document.addEventListener("keypress", scrollContainerByKey);
    });
    onUnmounted(() => {
      document.removeEventListener("keypress", scrollContainerByKey);
    });
  }

  //  autoScroll自动跳转
  if (autoScroll) {
    let oldFirstScoreContainer: HTMLDivElement | null = null;
    watch(
      () => markStore.currentTask,
      () => {
        if (markStore.setting.autoScroll) {
          // 给任务清理和动画留一点时间
          oldFirstScoreContainer =
            document.querySelector<HTMLDivElement>(".score-container");
          oldFirstScoreContainer?.scrollIntoView({ behavior: "smooth" });
          addTimeout(scrollToFirstScore, 1000);
        } else {
          const container = getContainer();
          container?.scrollTo({ top: 0, left: 0, behavior: "smooth" });
        }
      }
    );
  }

  function scrollToFirstScore() {
    if (markStore.renderLock) {
      window.requestAnimationFrame(scrollToFirstScore);
    }
    addTimeout(() => {
      const firstScore =
        document.querySelector<HTMLDivElement>(".score-container");
      firstScore?.scrollIntoView({ behavior: "smooth" });
    }, 1000);
  }

  // 放大缩小和之后的滚动
  const answerPaperScale = computed(() => {
    const scale = markStore.setting.uiSetting["answer.paper.scale"];
    return scale * 100 + "%";
  });

  const answerPaperFontSize = computed(() => {
    const scale = markStore.setting.uiSetting["answer.paper.scale"];
    return scale * 14 + "px";
  });

  watch(
    () => markStore.setting.uiSetting["answer.paper.scale"],
    () => {
      // 放大、缩小不影响页面之前的滚动条定位
      let percentWidth = 0;
      let percentTop = 0;
      const container = getContainer();
      if (container) {
        const { scrollLeft, scrollTop, scrollWidth, scrollHeight } = container;
        percentWidth = scrollLeft / scrollWidth;
        percentTop = scrollTop / scrollHeight;
      }

      addTimeout(() => {
        if (!container) return;

        const { scrollWidth, scrollHeight } = container;
        container.scrollTo({
          left: scrollWidth * percentWidth,
          top: scrollHeight * percentTop,
        });
      }, 10);
    }
  );

  return { answerPaperScale, answerPaperFontSize };
}
