import { SliceImage, SpecialTag, Track, Question, SplitConfig } from "@/types";
import { useMarkStore } from "@/store";
import { getTrackId } from "@/utils/utils";
import { message } from "ant-design-vue";

export default function useMakeTrack() {
  const markStore = useMarkStore();

  // 标记分数轨迹
  function makeScoreTrack(event: MouseEvent, item: SliceImage) {
    if (
      !markStore.currentQuestion ||
      markStore.currentQuestion.problem ||
      typeof markStore.currentScore === "undefined"
    )
      return;

    const target = event.target as HTMLImageElement;
    const track: Track = {
      id: getTrackId({
        mainNumber: markStore.currentQuestion?.mainNumber,
        subNumber: markStore.currentQuestion?.subNumber,
      }),
      mainNumber: markStore.currentQuestion?.mainNumber,
      subNumber: markStore.currentQuestion?.subNumber,
      score: markStore.currentScore,
      unanswered: Object.is(markStore.currentScore, -0),
      offsetIndex: item.indexInSliceUrls,
      // 新增轨迹时会校验轨迹是否距离边缘过近，使用round产生的误差可以忽略，不会发生超出边界的问题
      offsetX: Math.round(
        event.offsetX * (target.naturalWidth / target.width) + item.dx
      ),
      offsetY: Math.round(
        event.offsetY * (target.naturalHeight / target.height) + item.dy
      ),
      number: -1,
    };

    if (track.offsetX > item.effectiveWidth + item.dx) {
      console.log("不在有效宽度内，轨迹不生效");
      return;
    }
    if (
      item.markerTrackList.some((t) => {
        return (
          Math.pow(Math.abs(t.offsetX - track.offsetX), 2) +
            Math.pow(Math.abs(t.offsetY - track.offsetY), 2) <
          500
        );
      })
    ) {
      console.log("两个轨迹相距过近");
      return;
    }
    // 是否保留当前的轨迹分
    const questionScore =
      markStore.currentTask &&
      markStore.currentQuestion &&
      markStore.currentTask.markResult.scoreList[
        markStore.currentQuestion.__index
      ];
    const ifKeepScore =
      Math.round(
        markStore.currentQuestion.maxScore * 1000 -
          (questionScore || 0) * 1000 -
          markStore.currentScore * 2 * 1000
      ) / 1000;
    if (ifKeepScore < 0 && markStore.currentScore > 0) {
      markStore.currentScore = undefined;
    }
    const markResult = markStore.currentTaskEnsured.markResult;
    const maxNumber =
      markResult.markerTrackList.length === 0
        ? 0
        : Math.max(...markResult.markerTrackList.map((t) => t.number));
    track.number = maxNumber + 1;

    markResult.markerTrackList = [...markResult.markerTrackList, track];
    const { __index, mainNumber, subNumber } = markStore.currentQuestion;
    markResult.scoreList[__index] =
      markResult.markerTrackList
        .filter((t) => t.mainNumber === mainNumber && t.subNumber === subNumber)
        .map((t) => t.score)
        .reduce((acc, v) => (acc += Math.round(v * 1000)), 0) / 1000;
    item.markerTrackList.push(track);
  }

  // 标记特殊轨迹 （特殊轨迹包括：文字、线条、圆圈）
  function makeSpecialTagTrack(event: MouseEvent, item: SliceImage) {
    if (
      !markStore.currentTask ||
      typeof markStore.currentSpecialTag === "undefined"
    )
      return;

    const target = event.target as HTMLImageElement;
    const track: SpecialTag = {
      id: getTrackId({
        mainNumber: markStore.currentQuestion?.mainNumber,
        subNumber: markStore.currentQuestion?.subNumber,
      }),
      mainNumber: markStore.currentQuestion?.mainNumber,
      subNumber: markStore.currentQuestion?.subNumber,
      tagName: markStore.currentSpecialTag,
      tagType: markStore.currentSpecialTagType,
      offsetIndex: item.indexInSliceUrls,
      offsetX: Math.round(
        event.offsetX * (target.naturalWidth / target.width) + item.dx
      ),
      offsetY: Math.round(
        event.offsetY * (target.naturalHeight / target.height) + item.dy
      ),
    };

    if (
      item.tagList.some((t) => {
        return (
          Math.pow(Math.abs(t.offsetX - track.offsetX), 2) +
            Math.pow(Math.abs(t.offsetY - track.offsetY), 2) <
          500
        );
      })
    ) {
      console.log("两个轨迹相距过近");
      return;
    }
    markStore.currentTaskEnsured.markResult.markerTagList.push(track);
    item.tagList.push(track);
  }

  function checkTrackValidate(item: SliceImage): boolean {
    if (!markStore.currentQuestion) return true;

    const valid = (markStore.currentQuestion.picList || []).some(
      (area) =>
        `${area.i}_${area.x}_${area.y}_${area.w}_${area.h}` === item.sliceId
    );

    return valid;
  }

  // 标记轨迹
  function makeTrack(event: MouseEvent, item: SliceImage) {
    if (
      item.sliceId &&
      markStore.currentQuestion &&
      !checkTrackValidate(item)
    ) {
      void message.warning({
        content: "请在该题对应的答题区域里给分",
        duration: 3,
      });
      return;
    }
    if (markStore.currentSpecialTagType) {
      makeSpecialTagTrack(event, item);
      if (markStore.currentSpecialTagType === "TEXT") {
        markStore.currentSpecialTag = undefined;
        markStore.currentSpecialTagType = undefined;
      }
    } else {
      makeScoreTrack(event, item);
    }
  }

  function getRandomPosition(area: SplitConfig) {
    const { x, y, w, h } = area;
    return {
      x: x + w / 4 + (Math.random() * w) / 2,
      y: y + h / 4 + (Math.random() * h) / 2,
    };
  }

  function checkPointInArea(
    point: { x: number; y: number },
    area: { x: number; y: number; w: number; h: number }
  ): boolean {
    return (
      point.x >= area.x &&
      point.x <= area.x + area.w &&
      point.y >= area.y &&
      point.y <= area.y + area.h
    );
  }

  // 标记普通模式轨迹
  function makeCommonTrack(question: Question, score: number) {
    if (!markStore.currentTask || question.problem) return;

    const qno = `${question.mainNumber}_${question.subNumber}`;
    markStore.currentTaskModifyQuestion[qno] = true;

    markStore.currentQuestion = question;
    const markResult = markStore.currentTaskEnsured.markResult;
    markResult.scoreList[question.__index] = score;
    const questionTracks = markResult.markerTrackList.filter(
      (t) =>
        t.mainNumber === question.mainNumber &&
        t.subNumber === question.subNumber
    );
    // 多于一个轨迹时，删除全部轨迹，重新生成轨迹
    if (questionTracks.length > 1) {
      markResult.markerTrackList = markResult.markerTrackList.filter(
        (t) =>
          t.mainNumber !== question.mainNumber ||
          t.subNumber !== question.subNumber
      );
      markStore.sliceImagesWithTrackList.forEach((item) => {
        item.markerTrackList = item.markerTrackList.filter(
          (t) =>
            t.mainNumber !== question.mainNumber ||
            t.subNumber !== question.subNumber
        );
      });
    }

    const questionTrack = questionTracks.length > 1 ? null : questionTracks[0];
    // 存在轨迹时直接使用已有的轨迹位置
    if (questionTrack) {
      questionTrack.score = score;
      // 更新渲染轨迹中的分值
      let viewTrack = null;
      markStore.sliceImagesWithTrackList.forEach((item) => {
        if (viewTrack || item.indexInSliceUrls !== questionTrack.offsetIndex)
          return;
        viewTrack = item.markerTrackList.find((t) => {
          return t.id === questionTrack.id;
        });
        if (!viewTrack) return;
        viewTrack.score = score;
      });
      return;
    }

    // 通过题目设置的第一个评卷区，随机生成一个轨迹
    const { picList, qeustionPicList } = question;
    const area = qeustionPicList ? qeustionPicList[0] : picList[0] || null;
    if (!area) return;

    const imageSize = markStore.currentTaskSliceImages[area.i];
    if (!imageSize) return;

    // 不存在时，在第一个评卷区的中心50%的区域随机一个点
    const { x, y } = getRandomPosition(area);
    const track: Track = {
      id: getTrackId({
        mainNumber: question.mainNumber,
        subNumber: question.subNumber,
      }),
      mainNumber: question.mainNumber,
      subNumber: question.subNumber,
      score,
      unanswered: false,
      offsetIndex: area.i,
      offsetX: imageSize.width * x,
      offsetY: imageSize.height * y,
      number: -1,
    };
    markResult.markerTrackList.push(track);
    // 将轨迹更新到渲染轨迹中
    markStore.sliceImagesWithTrackList.forEach((item) => {
      if (item.indexInSliceUrls !== area.i) return;

      if (
        checkPointInArea(
          { x: track.offsetX, y: track.offsetY },
          {
            x: item.dx,
            y: item.dy,
            w: item.sliceImageWidth,
            h: item.sliceImageHeight,
          }
        )
      ) {
        item.markerTrackList.push(track);
      }
    });
  }

  return {
    makeScoreTrack,
    makeSpecialTagTrack,
    makeTrack,
    makeCommonTrack,
  };
}
