import { defineStore } from "pinia";
import { Setting, MarkStore, AdminPageSetting, Task } from "@/types";

const useMarkStore = defineStore("mark", {
  state: (): MarkStore => ({
    setting: {
      examType: "SCAN_IMAGE",
      mode: "TRACK",
      questionModel: "MULTI",
      forceMode: false,
      sheetView: false,
      sheetConfig: [],
      enableAllZero: false,
      fileServer: "",
      userName: "",
      subject: <Setting["subject"]>{},
      forceSpecialTag: false,
      uiSetting: {
        "answer.paper.scale": 1,
        "score.board.collapse": false,
        "normal.mode": "mouse",
        "score.fontSize.scale": 1,
        "paper.modal": false,
        "answer.modal": false,
        "minimap.modal": false,
        "specialTag.modal": false,
        "shortCut.modal": false,
      },
      statusValue: null,
      problemTypes: [],
      topCount: 0,
      splitConfig: [],
      prefetchCount: 3,
      startTime: 0,
      endTime: 0,
      selective: false,
      autoScroll: false,
      enableSplit: true,
      showObjectiveScore: false,
    },
    status: [],
    curStatus: {},
    tasks: [],
    currentTask: undefined,
    currentTaskSliceImages: undefined,
    sliceImagesWithTrackList: [],
    // 问题卷
    problemModalVisible: false,
    // 单题阅模式下，选择试题模态框是否可见
    selectMarkingQuestionModalVisible: false,
    // 主观题检查时，缓存已经修改过的试题
    currentTaskModifyQuestion: {},
    currentQuestion: undefined,
    currentScore: undefined,
    currentSpecialTag: undefined,
    currentSpecialTagType: undefined,
    historyOpen: false,
    historyTasks: [],
    removeScoreTracks: [],
    focusTracks: [],
    minimapScrollToX: 0,
    minimapScrollToY: 0,
    allPaperModal: false,
    sheetViewModal: false,
    globalMask: false,
    // 任务渲染锁状态
    renderLock: false,
    message: null,
  }),

  getters: {
    info(state: MarkStore): MarkStore {
      return { ...state };
    },
    /** 获得statusValue的中文名 */
    getStatusValueName(state: MarkStore): string {
      const st = state.setting.statusValue;
      if (!st) return "";
      if (st === "FORMAL") return "正评";
      if (st === "TRIAL") return "试评";
      return "";
    },
    /** 当前任务。确保不为空，需在上文已经检查过 store.currentTask 不为空 */
    currentTaskEnsured(state: MarkStore): Task {
      return state.currentTask;
    },
    /** 是否是评卷端的轨迹模式 */
    isTrackMode(state: MarkStore): boolean {
      return state.setting.mode && state.setting.mode === "TRACK";
    },
    isSingelQuestionModel(state: MarkStore): boolean {
      return state.setting.questionModel === "SINGLE";
    },
    modeName(state: MarkStore): string {
      return state.setting.mode === "TRACK" ? "轨迹模式" : "普通模式";
    },
    /* 是否是扫描阅卷 */
    isScanImage(state: MarkStore): boolean {
      return state.setting.examType === "SCAN_IMAGE";
    },
    isMultiMedia(state: MarkStore): boolean {
      return state.setting.examType === "MULTI_MEDIA";
    },
    /* 返回正在评卷的状态 '' | 回评 | 打回 */
    getMarkStatus(state: MarkStore): string {
      if (!state.currentTask) return "";
      if (state.currentTask.previous) return "回评";
      if (state.currentTask.rejected) return "打回";

      return state.getStatusValueName;
    },
    isScoreBoardCollapsed(state: MarkStore): boolean {
      return state.setting.uiSetting["score.board.collapse"];
    },
    isScoreBoardVisible(state: MarkStore): boolean {
      return !state.setting.uiSetting["score.board.collapse"];
    },
    toggleKeyMouse(state: MarkStore): void {
      state.setting.uiSetting["normal.mode"] =
        state.setting.uiSetting["normal.mode"] === "keyboard"
          ? "mouse"
          : "keyboard";
    },
    answerUrl(state: MarkStore): string {
      const answerUrl = state.setting.subject.answerUrl || [];
      if (answerUrl.length === 0) return "";

      return (
        answerUrl.filter((u) => u.paperType === state.currentTask?.paperType)[0]
          ?.url || ""
      );
    },
    paperUrl(state: MarkStore): string {
      const paperUrl = state.setting.subject.paperUrl || [];
      if (paperUrl.length === 0) return "";
      return (
        paperUrl.filter((u) => u.paperType === state.currentTask?.paperType)[0]
          ?.url || ""
      );
    },
  },

  actions: {
    setInfo(partial: Partial<MarkStore>) {
      this.$patch(partial);
    },
    resetInfo() {
      this.$reset();
    },
    initSetting(adminPageSetting: AdminPageSetting): void {
      this.setting = {
        ...this.setting,
        ...adminPageSetting,
        mode: "COMMON" as Setting["mode"],
        uiSetting: {
          "answer.paper.scale": 1,
          "score.board.collapse": false,
          "normal.mode": "keyboard",
          "score.fontSize.scale": 1,
        } as Setting["uiSetting"],
      };

      const fileServer = this.setting.fileServer;
      if (this.setting.subject?.answerUrl?.url) {
        this.setting.subject.answerUrl.url =
          fileServer + this.setting.subject?.answerUrl.url;
      }
      if (this.setting.subject?.paperUrl?.url) {
        this.setting.subject.paperUrl.url =
          fileServer + this.setting.subject?.paperUrl.url;
      }
    },
    toggleHistory(): void {
      this.historyOpen = !this.historyOpen;
    },
    toggleScoreBoard(): void {
      this.setting.uiSetting["score.board.collapse"] =
        !this.setting.uiSetting["score.board.collapse"];
    },
  },
  persist: {
    storage: sessionStorage,
  },
});

export default useMarkStore;
